@@ -44,10 +44,14 @@
 
 typedef struct _GeditCloseConfirmationDialog 		GeditCloseConfirmationDialog;
 typedef struct _GeditCloseConfirmationDialogClass 	GeditCloseConfirmationDialogClass;
+typedef struct _GeditCloseConfirmationDialogPrivate 	GeditCloseConfirmationDialogPrivate;
 
 struct _GeditCloseConfirmationDialog 
 {
 	GtkDialog parent;
+
+	/*< private > */
+	GeditCloseConfirmationDialogPrivate *priv;
 };
 
 struct _GeditCloseConfirmationDialogClass 
@@ -66,7 +70,7 @@ GtkWidget 	*gedit_close_confirmation_dialog_new_single 		(GtkWindow     *parent,
 
 const GList	*gedit_close_confirmation_dialog_get_unsaved_documents  (GeditCloseConfirmationDialog *dlg);
 
-const GList	*gedit_close_confirmation_dialog_get_selected_documents	(GeditCloseConfirmationDialog *dlg);
+GList		*gedit_close_confirmation_dialog_get_selected_documents	(GeditCloseConfirmationDialog *dlg);
 
 #endif /* __GEDIT_CLOSE_CONFIRMATION_DIALOG_H__ */
 
