@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document-loader.c 5323 2006-12-01 12:07:55Z pborelli $
+ * $Id: gedit-document-loader.c 5574 2007-03-13 17:40:04Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,8 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/mman.h>
+#include <signal.h>
+#include <setjmp.h>
 #include <string.h>
 #include <errno.h>
 
@@ -87,6 +89,7 @@ struct _GeditDocumentLoaderPrivate
 
 	gchar                    *buffer;
 
+	const GeditEncoding      *metadata_encoding;
 	const GeditEncoding      *auto_detected_encoding;
 
 	GError                   *error;
@@ -249,17 +252,37 @@ gedit_document_loader_new (GeditDocument *doc)
 	return dl;						  
 }
 
+static const GeditEncoding *
+get_metadata_encoding (const gchar *uri)
+{
+	const GeditEncoding *enc;
+	gchar *charset;
+
+	charset = gedit_metadata_manager_get (uri, "encoding");
+
+	if (charset == NULL)
+		return NULL;
+
+	enc = gedit_encoding_get_from_charset (charset);
+
+	g_free (charset);
+
+	return enc;
+}
+
 static void
 insert_text_in_document (GeditDocumentLoader *loader,
 			 const gchar         *text,
 			 gint                 len)
 {
+	g_return_if_fail (text != NULL);
+
 	gtk_source_buffer_begin_not_undoable_action (
 				GTK_SOURCE_BUFFER (loader->priv->document));
 
 	/* If the last char is a newline, don't add it to the buffer
 	(otherwise GtkTextView shows it as an empty line). See bug #324942. */
-	if (text[len-1] == '\n')
+	if ((len > 0) && (text[len-1] == '\n'))
 		len--;
 
 	/* Insert text in the buffer */
@@ -316,29 +339,17 @@ update_document_contents (GeditDocumentLoader  *loader,
 			/* Autodetecting the encoding: first try with the encoding
 			stored in the metadata, if any */
 
-			const GeditEncoding *enc;
-			gchar *charset;
-
-			charset = gedit_metadata_manager_get (loader->priv->uri, "encoding");
-
-			if (charset != NULL)
+			if (loader->priv->metadata_encoding != NULL)
 			{
-				enc = gedit_encoding_get_from_charset (charset);
+				converted_text = gedit_convert_to_utf8 (
+								file_contents,
+								file_size,
+								&loader->priv->metadata_encoding,
+								&new_len,
+								NULL);
 
-				if (enc != NULL)
-				{	
-					converted_text = gedit_convert_to_utf8 (
-									file_contents,
-									file_size,
-									&enc,
-									&new_len,
-									NULL);
-
-					if (converted_text != NULL)
-						loader->priv->auto_detected_encoding = enc;
-				}
-
-				g_free (charset);
+				if (converted_text != NULL)
+					loader->priv->auto_detected_encoding = loader->priv->metadata_encoding;
 			}
 		}
 
@@ -500,6 +511,18 @@ load_completed_or_failed (GeditDocumentLoader *loader)
 
 #define MAX_MIME_SNIFF_SIZE 4096
 
+static sigjmp_buf mmap_env;
+static struct sigaction old_sigbusact;
+
+/* When we access the mmapped data we need to handle
+ * SIGBUS signal, which is emitted if there was an
+ * I/O error or if the file was truncated */
+static void
+mmap_sigbus_handler (int signo)
+{
+	siglongjmp (mmap_env, 0);
+}
+
 static gboolean
 load_local_file_real (GeditDocumentLoader *loader)
 {
@@ -552,12 +575,16 @@ load_local_file_real (GeditDocumentLoader *loader)
 		if (loader->priv->encoding == NULL)
 			loader->priv->auto_detected_encoding = gedit_encoding_get_current ();
 
+		/* clear the contents in case we are reverting */
+		insert_text_in_document (loader, "", 0);
+
 		/* guessing the mime from the filename is up to the caller */
 	}
 	else
 	{
 		gchar *mapped_file;
 		const gchar *mime_type;
+		struct sigaction sigbusact;
 
 		/* CHECK: should we lock the file */		
 		mapped_file = mmap (0, /* start */
@@ -566,7 +593,7 @@ load_local_file_real (GeditDocumentLoader *loader)
 				    MAP_PRIVATE, /* flags */
 				    loader->priv->fd,
 				    0 /* offset */);
-				    
+
 		if (mapped_file == MAP_FAILED)
 		{
 			gedit_debug_message (DEBUG_LOADER, "mmap failed");
@@ -580,9 +607,37 @@ load_local_file_real (GeditDocumentLoader *loader)
 
 			goto done;
 		}
-		
+
+		/* prepare to handle the SIGBUS signal which
+		 * may be fired when accessing the mmapped
+		 * data in case of I/O error.
+		 * See bug #354046.
+		 */
+		sigbusact.sa_handler = mmap_sigbus_handler;
+		sigemptyset (&sigbusact.sa_mask);
+		sigbusact.sa_flags = SA_RESETHAND;
+		sigaction (SIGBUS, &sigbusact, &old_sigbusact );
+
+		if (sigsetjmp (mmap_env, 1) != 0)
+		{
+			gedit_debug_message (DEBUG_LOADER, "SIGBUS during mmap");
+
+			g_set_error (&loader->priv->error,
+				     GEDIT_DOCUMENT_ERROR,
+				     GNOME_VFS_ERROR_IO,
+				     gnome_vfs_result_to_string (GNOME_VFS_ERROR_IO));
+
+			ret = munmap (mapped_file, loader->priv->info->size);
+			if (ret != 0)
+				g_warning ("File '%s' has not been correctly unmapped: %s",
+					   loader->priv->uri,
+					   strerror (errno));
+
+			goto done;
+		}
+
 		loader->priv->bytes_read = loader->priv->info->size;
-		
+
 		if (!update_document_contents (loader,
 					       mapped_file,
 					       loader->priv->info->size,
@@ -597,6 +652,9 @@ load_local_file_real (GeditDocumentLoader *loader)
 			goto done;
 		}
 
+		/* restore the default sigbus handler */
+		sigaction (SIGBUS, &old_sigbusact, 0);
+
 		mime_type = gnome_vfs_get_mime_type_for_name_and_data (loader->priv->local_file_name,
 				mapped_file,
 				MIN (loader->priv->bytes_read, MAX_MIME_SNIFF_SIZE));
@@ -616,6 +674,7 @@ load_local_file_real (GeditDocumentLoader *loader)
 				   strerror (errno));
 	}
 
+ done:
 	ret = close (loader->priv->fd);
 
 	if (ret != 0)
@@ -625,7 +684,6 @@ load_local_file_real (GeditDocumentLoader *loader)
 
 	loader->priv->fd = -1;
 
- done:
 	load_completed_or_failed (loader);
 	
 	return FALSE;
@@ -970,6 +1028,11 @@ gedit_document_loader_load (GeditDocumentLoader *loader,
 
 	loader->priv->encoding = encoding;
 
+	if (encoding == NULL)
+	{
+		loader->priv->metadata_encoding = get_metadata_encoding (uri);
+	}
+
 	loader->priv->uri = g_strdup (uri);
 
 	local_path = gnome_vfs_get_local_path_from_uri (uri);
