@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.c,v 1.133 2006/08/07 18:04:06 pborelli Exp $
+ * $Id: gedit-document.c 5428 2007-01-08 22:02:42Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,7 +44,6 @@
 #include "gedit-document.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
-#include "gedit-convert.h"
 #include "gedit-metadata-manager.h"
 #include "gedit-languages-manager.h"
 #include "gedit-document-loader.h"
@@ -712,7 +711,7 @@ set_uri (GeditDocument *doc,
 
 			if (strcmp (doc->priv->mime_type, "text/plain") != 0)
 			{
-				language = gtk_source_languages_manager_get_language_from_mime_type (
+				language = gedit_languages_manager_get_language_from_mime_type (
 							gedit_get_languages_manager (),
 							doc->priv->mime_type);
 			}
@@ -888,6 +887,40 @@ gedit_document_get_readonly (GeditDocument *doc)
 	return doc->priv->readonly;
 }
 
+gboolean
+_gedit_document_check_externally_modified (GeditDocument *doc)
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult result;
+	gint file_mtime;
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+
+	if (doc->priv->uri == NULL)
+	{
+		return FALSE;
+	}
+
+	info = gnome_vfs_file_info_new ();
+
+	result = gnome_vfs_get_file_info_uri (doc->priv->vfs_uri,
+					      info,
+					      GNOME_VFS_FILE_INFO_DEFAULT|
+					      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (result != GNOME_VFS_OK)
+	{
+		gnome_vfs_file_info_unref (info);
+
+		return FALSE;
+	}
+
+	file_mtime = info->mtime;
+
+	gnome_vfs_file_info_unref (info);
+
+	return (file_mtime > doc->priv->mtime);
+}
+
 static void
 reset_temp_loading_data (GeditDocument       *doc)
 {
@@ -962,10 +995,10 @@ document_loader_loaded (GeditDocumentLoader *loader,
 
 	/* special case creating a named new doc */
 	else if (doc->priv->create &&
-	         (error->code == GNOME_VFS_ERROR_NOT_FOUND))
+	         (error->code == GNOME_VFS_ERROR_NOT_FOUND) &&
+	         (gedit_utils_uri_has_file_scheme (doc->priv->uri)))
 	{
 		reset_temp_loading_data (doc);
-		// FIXME: do other stuff??
 
 		g_signal_emit (doc,
 			       document_signals[LOADED],
@@ -1203,6 +1236,18 @@ gedit_document_is_untitled (GeditDocument *doc)
 	return (doc->priv->uri == NULL);
 }
 
+gboolean
+gedit_document_is_local (GeditDocument *doc)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+
+	if (doc->priv->uri == NULL)
+	{
+		return FALSE;
+	}
+
+	return gedit_utils_uri_has_file_scheme (doc->priv->uri);
+}
 
 gboolean
 gedit_document_get_deleted (GeditDocument *doc)
