@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-panel.h,v 1.4 2006/01/19 10:33:30 pborelli Exp $
+ * $Id: gedit-panel.h 5244 2006-09-23 14:37:09Z pborelli $
  */
 
 #ifndef __GEDIT_PANEL_H__
@@ -118,6 +118,14 @@ GtkOrientation	 gedit_panel_get_orientation	(GeditPanel	*panel);
 gint		 gedit_panel_get_n_items	(GeditPanel	*panel);
 
 
+/*
+ * Non exported functions
+ */
+gint		 _gedit_panel_get_active_item_id	(GeditPanel	*panel);
+
+void		 _gedit_panel_set_active_item_by_id	(GeditPanel	*panel,
+							 gint		 id);
+
 G_END_DECLS
 
 #endif  /* __GEDIT_PANEL_H__  */
