@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-print-job-preview.c 4429 2005-12-12 17:28:04Z pborelli $
+ * $Id: gedit-print-job-preview.c 5473 2007-02-06 08:54:44Z pborelli $
  */
  
 /* This is a modified version of the file gnome-print-job-preview.f file
@@ -848,8 +848,8 @@ create_toplevel (GeditPrintJobPreview *mp)
 	i = gtk_separator_tool_item_new ();
 	gtk_widget_show (GTK_WIDGET (i));
 	gtk_toolbar_insert (GTK_TOOLBAR (tb), i, -1);
-		
-	i = gtk_tool_button_new (NULL, "_Close Preview");
+
+	i = gtk_tool_button_new (NULL, _("_Close Preview"));
 	gtk_tool_button_set_use_underline (GTK_TOOL_BUTTON (i), TRUE);
 	gtk_tool_item_set_is_important (i, TRUE);
 	gtk_tool_item_set_tooltip (i, GTK_TOOLBAR (tb)->tooltips, 
