@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit.c,v 1.144 2006/11/08 15:15:54 sfre Exp $
+ * $Id: gedit.c 5369 2006-12-17 15:41:19Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -214,14 +214,20 @@ on_message_received (const char *message,
 
 	/* header */
 	params = g_strsplit (commands[0], "\t", 4);
-	startup_timestamp = atoi (params[0]); /* CHECK if this is safe */
-	display_name = g_strdup (params[1]);
+	startup_timestamp = atoi (params[0]);
+	display_name = params[1];
 	screen_number = atoi (params[2]);
 	workspace = atoi (params[3]);
 
 	display = display_open_if_needed (display_name);
+	if (display == NULL)
+	{
+		g_warning ("Could not open display %s\n", display_name);
+		g_strfreev (params);
+		goto out;
+	}
+
 	screen = gdk_display_get_screen (display, screen_number);
-	g_free (display_name);
 
 	g_strfreev (params);
 
@@ -266,8 +272,6 @@ on_message_received (const char *message,
 		g_strfreev (params);
 	}
 
-	g_strfreev (commands);
-
 	/* execute the commands */
 
 	app = gedit_app_get_default ();
@@ -327,6 +331,9 @@ on_message_received (const char *message,
 	g_slist_free (file_list);
 	file_list = NULL;
 
+ out:
+	g_strfreev (commands);
+
 	new_window_option = FALSE;
 	new_document_option = FALSE;
 	encoding = NULL;
