@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-taglist-plugin-parser.c,v 1.13 2006/12/09 13:26:06 paolo Exp $
+ * $Id: gedit-taglist-plugin-parser.c 5583 2007-03-24 13:21:06Z paolo $
  */
 
 /* FIXME: we should rewrite the parser to avoid using DOM */
@@ -578,7 +578,8 @@ parse_taglist_dir (const gchar *dir)
 	
 	while ((e = readdir (d)) != NULL)
 	{
-		if (strncmp (e->d_name + strlen (e->d_name) - 5, ".tags", 5) == 0)
+		if (strncmp (e->d_name + strlen (e->d_name) - 5, ".tags", 5) == 0 ||
+		    strncmp (e->d_name + strlen (e->d_name) - 8, ".tags.gz", 8) == 0)
 		{
 			gchar *tags_file = g_strconcat (dir, e->d_name, NULL);
 			parse_taglist_file (tags_file);
